<html>
<head>
  <script type="text/javascript" src="/old_jsi18n/app1/"></script>
  <script type="text/javascript" src="/old_jsi18n/app2/"></script>
<body>
  <p id="app1string">
    <script type="text/javascript">
      document.write(gettext('this app1 string is to be translated'))
    </script>
  </p>
  <p id="app2string">
    <script type="text/javascript">
      document.write(gettext('this app2 string is to be translated'))
    </script>
  </p>
</body>
</html>
